import pycurl
import xmltodict
import time
import socket
import transform
import platform
import traceback

import logging
import thread
import datetime
from datetime import timedelta
from ardi.driver import resultstore

class SQLSource:
    def __init__(self,db,driver):
        self.db = db
        self.driver = driver
        self.core = driver
        self.filter = ""
        self.narrowthreshold = 20
        self.server = "mysql"

    def ConnectionParameters(self,host,username,password,database):
        self.host = host
        self.username = username
        self.password = password
        self.database = database

    def TableParameters(self,stamp, lookup, value,table,filter=""):
        self.table = table
        self.stamp = stamp
        self.lookup = lookup
        self.value = value
        self.filter = filter

    def Reconnect(self):
        return self.driver.Connect()
        pass

    def Fetch(self, sd, ed, function, grain, points):
        Results = resultstore.ResultStore()

        #print "Database Commit"
        #Commit the database
        try:
            self.db.commit()
        except:
            self.driver.Connect()
            self.db = self.driver.db

        #Figure out effective grain...

        diff = ed - sd

        #secs = diff.seconds + (diff.minutes * 60) + (diff.hours * (60*60)) + (diff.days * (60*60*24))
        secs = diff.total_seconds()

        itemcount = int(secs) / int(grain)
        if int(grain) < 0:
            #print "Dealing With Negative Grain"
            itemcount = -int(grain)
            grain = secs / (itemcount)

        #print "Taking " + str(itemcount) + " samples with " + str(grain) + " second intervals"

        #print "Requesting data before start..."

        #Now, get the PREVIOUS values so we have something to interpolate with...
        if self.server == "mysql":
            topoffquery = "SELECT " + self.stamp + ", " + self.lookup + "," + self.value +" FROM ( SELECT "
            if self.server == "mssql":
                topoffquery += "TOP (100) PERCENT "
            topoffquery += "* "		
            topoffquery += "FROM " + self.table + " WHERE " + self.stamp + " <= '" + str(sd) + "'"
            if self.filter != "":
                topoffquery += " AND (" + self.filter + ")"

            if len(points) < self.narrowthreshold:
                topoffquery += " AND (" + self.lookup + " IN ("
                x=0
                for p in points:
                    if x > 0:
                        topoffquery += ","
                    topoffquery += "'" + p + "'"
                    x = x + 1
                topoffquery += "))"

            topoffquery += " ORDER BY " + self.stamp + " DESC) AS t1 GROUP BY " + self.lookup
        if self.server == "mssql":
            topoffquery = "SELECT TOP 1 WITH TIES " + self.stamp +"," + self.lookup + "," + self.value + " FROM " + self.table + " WHERE " + self.stamp + " <= '" + str(sd) + "' "
            if self.filter != "":
                topoffquery += "AND (" + self.filter + ") "
            if len(points) < self.narrowthreshold:
                topoffquery += " AND (" + self.lookup + " IN ("
                x=0
                for p in points:
                    if x > 0:
                        topoffquery += ","
                    topoffquery += "'" + p + "'"
                    x = x + 1
                topoffquery += "))"
            topoffquery += "ORDER BY ROW_NUMBER() OVER (PARTITION by " + self.lookup + " ORDER BY " + self.stamp + " DESC)"

        self.driver.core.logger.debug("Pre-Fetch Query: " + topoffquery)
        try:
            csr = self.db.cursor()
            csr.execute(topoffquery);
        except:
            traceback.print_exc()
            self.driver.Connect()
            self.db = self.driver.db
            try:
                csr = self.db.cursor()
                csr.execute(topoffquery);
            except:
                return ""

        for row in csr:
        	if row[1] in points:
        	    #print str(row)
        	    Results.Record(str(row[1]),row[0],row[2])

        csr.close()

        func = "AVG"
        if function == "min":
        	func = "MIN"
        if function == "max":
        	func = "MAX"

        #Get the intermediate values...
        self.query = "SELECT " + self.stamp + "," + self.lookup + "," + self.value + " FROM " + self.table + " WHERE "
        if grain != 0:
        	self.query = "SELECT "
        	calc = "DATE_ADD('" + str(sd) + "',INTERVAL (CEIL(TIMESTAMPDIFF(SECOND,'" + str(sd) + "'," + self.stamp + ") / " + str(grain) + ") * " + str(grain) + ") SECOND)"
        	if self.server == 'mssql':
        	    calc = "DATEADD(s,(CEILING(DATEDIFF(second,'" + str(sd) + "'," + self.stamp + ") / " + str(grain) + ") * " + str(grain) + "),'" + str(sd) + "')"    

        	self.query += calc
        	self.query += " as ostamp, " + self.lookup + " as lookup,"+ func + "(" + self.value + ") as value" + " FROM " + self.table + " WHERE "

        if self.filter != "":
        	self.query += "( " + self.filter + " ) AND "
        	
        self.query += "( " + self.stamp + " >= '" + str(sd) + "' AND " + self.stamp + " <= '" + str(ed) + "' )"

        #Filter for point list...

        if len(points) < self.narrowthreshold:
            self.query += " AND (" + self.lookup + " IN ("
            x=0
            for p in points:
             	if x > 0:
             		self.query += ","
             	self.query += "'" + p + "'"
             	x = x + 1
            self.query += "))"
            
        if grain != 0:
            if self.server == "mssql":
                self.query += " GROUP BY " + calc + ", " + self.lookup + " ORDER BY ostamp,lookup"
            else:
                self.query += " GROUP BY ostamp, lookup ORDER BY ostamp,lookup"
        else:
        	self.query += " ORDER BY " + self.stamp + " ASC"

        #print "Submitting Query: " + self.query
        self.driver.core.logger.debug("Main Query: " + self.query)

        try:
        	#print self.query
        	csr = self.db.cursor()			

        	csr.execute(self.query)		
        	for row in csr:
        		if row[1] in points:
        			#print str(row)
        			Results.Record(str(row[1]),row[0],row[2])

        	csr.close()
        except:
        	traceback.print_exc()
        	pass

        #Get the NEXT values so we have something to interpolate into...
        #print "Getting data from AFTER the range..."
        if self.server == "mysql":
            topoffquery = "SELECT " + self.stamp + ", " + self.lookup + "," + self.value +" FROM ( SELECT * "		
            topoffquery += "FROM " + self.table + " WHERE " + self.stamp + " >= '" + str(ed) + "'"
            if self.filter != "":
                topoffquery += " AND (" + self.filter + ")"

            if len(points) < self.narrowthreshold:
                topoffquery += " AND (" + self.lookup + " IN ("
                x=0
                for p in points:
                    if x > 0:
                        topoffquery += ","
                    topoffquery += "'" + p + "'"
                    x = x + 1
                topoffquery += "))"

            topoffquery += " ORDER BY " + self.stamp + " ASC) AS t1 GROUP BY " + self.lookup
        if self.server == "mssql":
            topoffquery = "SELECT TOP 1 WITH TIES " + self.stamp +"," + self.lookup + "," + self.value + " FROM " + self.table + " WHERE " + self.stamp + " >= '" + str(ed) + "' "
            if self.filter != "":
                topoffquery += "AND (" + self.filter + ") "
            if len(points) < self.narrowthreshold:
                topoffquery += " AND (" + self.lookup + " IN ("
                x=0
                for p in points:
                    if x > 0:
                        topoffquery += ","
                    topoffquery += "'" + p + "'"
                    x = x + 1
                topoffquery += "))"
            topoffquery += "ORDER BY ROW_NUMBER() OVER (PARTITION by " + self.lookup + " ORDER BY " + self.stamp + " ASC)"

        self.driver.core.logger.debug("Post-Fetch Query: " + topoffquery)
        try:	
        	csr = self.db.cursor()
        	csr.execute(topoffquery);
        
        	#OK - same deal, but this time get the NEXT row value from each...

        	for row in csr:
        	    if row[1] in points:
        	        print str(row)
        	        Results.Record(str(row[1]),row[0],row[2])
        	    pass

        	csr.close()
        except:
        	pass

        try:
        	self.db.commit()
        except:
        	pass

        #Results.Interpolate(grain,itemcount)
        #Results.Fit(sd,ed)
        #Results.Interpolate(grain,itemcount)
        #return Results.Output(self.core,points)
        v = Results.Prepare(function,grain,sd,ed,self.core,points)
        return v